@@ -828,87 +828,6 @@ bfd_set_start_address (abfd, vma)
   return true;
 }
 
-/*
-FUNCTION
-	bfd_get_mtime
-
-SYNOPSIS
-	long bfd_get_mtime(bfd *abfd);
-
-DESCRIPTION
-	Return the file modification time (as read from the file system, or
-	from the archive header for archive members).
-
-*/
-
-long
-bfd_get_mtime (abfd)
-     bfd *abfd;
-{
-  FILE *fp;
-  struct stat buf;
-
-  if (abfd->mtime_set)
-    return abfd->mtime;
-
-  fp = bfd_cache_lookup (abfd);
-  if (0 != fstat (fileno (fp), &buf))
-    return 0;
-
-  abfd->mtime = buf.st_mtime;		/* Save value in case anyone wants it */
-  return buf.st_mtime;
-}
-
-/*
-FUNCTION
-	bfd_get_size
-
-SYNOPSIS
-	long bfd_get_size(bfd *abfd);
-
-DESCRIPTION
-	Return the file size (as read from file system) for the file
-	associated with BFD @var{abfd}.
-
-	The initial motivation for, and use of, this routine is not
-	so we can get the exact size of the object the BFD applies to, since
-	that might not be generally possible (archive members for example).
-	It would be ideal if someone could eventually modify
-	it so that such results were guaranteed.
-
-	Instead, we want to ask questions like "is this NNN byte sized
-	object I'm about to try read from file offset YYY reasonable?"
-	As as example of where we might do this, some object formats
-	use string tables for which the first <<sizeof (long)>> bytes of the
-	table contain the size of the table itself, including the size bytes.
-	If an application tries to read what it thinks is one of these
-	string tables, without some way to validate the size, and for
-	some reason the size is wrong (byte swapping error, wrong location
-	for the string table, etc.), the only clue is likely to be a read
-	error when it tries to read the table, or a "virtual memory
-	exhausted" error when it tries to allocate 15 bazillon bytes
-	of space for the 15 bazillon byte table it is about to read.
-	This function at least allows us to answer the quesion, "is the
-	size reasonable?".
-*/
-
-long
-bfd_get_size (abfd)
-     bfd *abfd;
-{
-  FILE *fp;
-  struct stat buf;
-
-  if ((abfd->flags & BFD_IN_MEMORY) != 0)
-    return ((struct bfd_in_memory *) abfd->iostream)->size;
-
-  fp = bfd_cache_lookup (abfd);
-  if (0 != fstat (fileno (fp), & buf))
-    return 0;
-
-  return buf.st_size;
-}
-
 /*
 FUNCTION
 	bfd_get_gp_size
@@ -1438,7 +1357,6 @@ bfd_preserve_save (abfd, preserve)
   preserve->tdata = abfd->tdata.any;
   preserve->arch_info = abfd->arch_info;
   preserve->flags = abfd->flags;
-
   preserve->sections = abfd->sections;
   preserve->section_tail = abfd->section_tail;
   preserve->section_count = abfd->section_count;
@@ -1449,8 +1367,7 @@ bfd_preserve_save (abfd, preserve)
 
   abfd->tdata.any = NULL;
   abfd->arch_info = &bfd_default_arch_struct;
-  abfd->flags = 0;
-
+  abfd->flags &= BFD_IN_MEMORY;
   abfd->sections = NULL;
   abfd->section_tail = &abfd->sections;
   abfd->section_count = 0;
@@ -1482,7 +1399,6 @@ bfd_preserve_restore (abfd, preserve)
   abfd->tdata.any = preserve->tdata;
   abfd->arch_info = preserve->arch_info;
   abfd->flags = preserve->flags;
-
   abfd->section_htab = preserve->section_htab;
   abfd->sections = preserve->sections;
   abfd->section_tail = preserve->section_tail;
