@@ -1,5 +1,5 @@
 /* BFD backend for MIPS BSD (a.out) binaries.
-   Copyright 1993, 1994, 1995, 1997, 1998, 1999, 2000, 2001
+   Copyright 1993, 1994, 1995, 1997, 1998, 1999, 2000, 2001, 2002
    Free Software Foundation, Inc.
    Written by Ralph Campbell.
 
@@ -36,7 +36,10 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 			    || (mtype) == M_MIPS1 || (mtype) == M_MIPS2)
 #define MY_symbol_leading_char '\0'
 
-#define MY(OP) CAT(mipsbsd_,OP)
+/* Do not "beautify" the CONCAT* macro args.  Traditional C will not
+   remove whitespace added here, and thus will fail to concatenate
+   the tokens.  */
+#define MY(OP) CONCAT2 (mipsbsd_,OP)
 
 #include "bfd.h"
 #include "sysdep.h"
@@ -46,13 +49,13 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #define SET_ARCH_MACH(ABFD, EXEC) \
   MY(set_arch_mach) (ABFD, N_MACHTYPE (EXEC)); \
   MY(choose_reloc_size) (ABFD);
-static void MY(set_arch_mach) PARAMS ((bfd *abfd, int machtype));
+static void MY(set_arch_mach) PARAMS ((bfd *abfd, unsigned long machtype));
 static void MY(choose_reloc_size) PARAMS ((bfd *abfd));
 
 #define MY_write_object_contents MY(write_object_contents)
 static boolean MY(write_object_contents) PARAMS ((bfd *abfd));
 
-/* We can't use MY(x) here because it leads to a recursive call to CAT
+/* We can't use MY(x) here because it leads to a recursive call to CONCAT2
    when expanded inside JUMP_TABLE.  */
 #define MY_bfd_reloc_type_lookup mipsbsd_reloc_howto_type_lookup
 #define MY_canonicalize_reloc mipsbsd_canonicalize_reloc
@@ -67,30 +70,33 @@ static boolean MY(write_object_contents) PARAMS ((bfd *abfd));
 
 #include "aout-target.h"
 
-static bfd_reloc_status_type mips_fix_jmp_addr PARAMS ((bfd *, arelent *, struct symbol_cache_entry *, PTR, asection *, bfd *));
-static reloc_howto_type *    MY(reloc_howto_type_lookup) PARAMS ((bfd *, bfd_reloc_code_real_type));
+static bfd_reloc_status_type mips_fix_jmp_addr
+  PARAMS ((bfd *, arelent *, struct symbol_cache_entry *, PTR, asection *,
+	   bfd *, char **));
+static reloc_howto_type *MY(reloc_howto_type_lookup)
+  PARAMS ((bfd *, bfd_reloc_code_real_type));
 
 long MY(canonicalize_reloc) PARAMS ((bfd *, sec_ptr, arelent **, asymbol **));
 
 static void
 MY(set_arch_mach) (abfd, machtype)
      bfd *abfd;
-     int machtype;
+     unsigned long machtype;
 {
   enum bfd_architecture arch;
-  long machine;
+  unsigned int machine;
 
   /* Determine the architecture and machine type of the object file.  */
   switch (machtype)
     {
     case M_MIPS1:
       arch = bfd_arch_mips;
-      machine = 3000;
+      machine = bfd_mach_mips3000;
       break;
 
     case M_MIPS2:
       arch = bfd_arch_mips;
-      machine = 4000;
+      machine = bfd_mach_mips4000;
       break;
 
     default:
@@ -158,8 +164,8 @@ MY (write_object_contents) (abfd)
     case bfd_arch_mips:
       switch (bfd_get_mach (abfd))
 	{
-	case 4000:
-	case 6000:
+	case bfd_mach_mips4000:
+	case bfd_mach_mips6000:
 	  N_SET_MACHTYPE (*execp, M_MIPS2);
 	  break;
 	default:
@@ -194,13 +200,15 @@ MY (write_object_contents) (abfd)
    program counter, then we need to signal an error.  */
 
 static bfd_reloc_status_type
-mips_fix_jmp_addr (abfd,reloc_entry,symbol,data,input_section,output_bfd)
+mips_fix_jmp_addr (abfd, reloc_entry, symbol, data, input_section, output_bfd,
+		   error_message)
      bfd *abfd ATTRIBUTE_UNUSED;
      arelent *reloc_entry;
      struct symbol_cache_entry *symbol;
      PTR data ATTRIBUTE_UNUSED;
      asection *input_section;
      bfd *output_bfd;
+     char **error_message ATTRIBUTE_UNUSED;
 {
   bfd_vma relocation, pc;
 
@@ -378,7 +386,7 @@ MY(canonicalize_reloc) (abfd, section, relptr, symbols)
   return section->reloc_count;
 }
 
-static CONST struct aout_backend_data MY(backend_data) = {
+static const struct aout_backend_data MY(backend_data) = {
   0,				/* zmagic contiguous */
   1,				/* text incl header */
   0,				/* entry is text address */
