@@ -23,30 +23,26 @@
   TODOs:
   ------
   
-  o .align cannot handle fill-data larger than 0xFF/8-bits
+  o .align cannot handle fill-data-width larger than 0xFF/8-bits. It
+    should be possible to define a 32-bits pattern.
 
   o .align fills all section with NOP's when used regardless if has
     been used in .text or .data. (However the .align is primarely
     intended used in .text sections. If you require something else,
     use .align <size>,0x00)
 
-  o .align: Implement a 'bu' insn if the number of nop's exeeds 4 within
-    the align frag. if(fragsize>4words) insert bu fragend+1 first. 
+  o .align: Implement a 'bu' insn if the number of nop's exeeds 4
+    within the align frag. if(fragsize>4words) insert bu fragend+1
+    first.
 
   o .usect if has symbol on previous line not implemented
 
   o .sym, .eos, .stag, .etag, .member not implemented
 
-  o Evaluation of constant floating point expressions (expr.c needs work!)
+  o Evaluation of constant floating point expressions (expr.c needs
+    work!)
 
-  o Warnings issued if parallel load of same register
-
-  o Support 'abc' constants?
-
-  o Support new opcodes and implement a silicon version switch (maybe -mpg)
-
-  o Disallow non-float registers in float instructions. Make as require
-    'fx' notation on floats, while 'rx' on the rest
+  o Support 'abc' constants (that is 0x616263)
 */
 
 #include <stdio.h>
@@ -77,8 +73,42 @@ static struct hash_control *c4x_op_hash = NULL;
 static struct hash_control *c4x_asg_hash = NULL;
 
 static unsigned int c4x_cpu = 0;	/* Default to TMS320C40.  */
+static unsigned int c4x_revision = 0;   /* CPU revision */
+static unsigned int c4x_idle2 = 0;      /* Idle2 support */
+static unsigned int c4x_lowpower = 0;   /* Lowpower support */
+static unsigned int c4x_enhanced = 0;   /* Enhanced opcode support */
 static unsigned int c4x_big_model = 0;	/* Default to small memory model.  */
 static unsigned int c4x_reg_args = 0;	/* Default to args passed on stack.  */
+static unsigned long c4x_oplevel = 0;   /* Opcode level */
+
+#define OPTION_CPU      'm'
+#define OPTION_BIG      (OPTION_MD_BASE + 1)
+#define OPTION_SMALL    (OPTION_MD_BASE + 2)
+#define OPTION_MEMPARM  (OPTION_MD_BASE + 3)
+#define OPTION_REGPARM  (OPTION_MD_BASE + 4)
+#define OPTION_IDLE2    (OPTION_MD_BASE + 5)
+#define OPTION_LOWPOWER (OPTION_MD_BASE + 6)
+#define OPTION_ENHANCED (OPTION_MD_BASE + 7)
+#define OPTION_REV      (OPTION_MD_BASE + 8)
+
+CONST char *md_shortopts = "bm:prs";
+struct option md_longopts[] =
+{
+  { "mcpu",   required_argument, NULL, OPTION_CPU },
+  { "mdsp",   required_argument, NULL, OPTION_CPU },
+  { "mbig",         no_argument, NULL, OPTION_BIG },
+  { "msmall",       no_argument, NULL, OPTION_SMALL },
+  { "mmemparm",     no_argument, NULL, OPTION_MEMPARM },
+  { "mregparm",     no_argument, NULL, OPTION_REGPARM },
+  { "midle2",       no_argument, NULL, OPTION_IDLE2 },
+  { "mlowpower",    no_argument, NULL, OPTION_LOWPOWER },
+  { "menhanced",    no_argument, NULL, OPTION_ENHANCED },
+  { "mrev",   required_argument, NULL, OPTION_REV },
+  { NULL, no_argument, NULL, 0 }
+};
+
+size_t md_longopts_size = sizeof (md_longopts);
+
 
 typedef enum
   {
@@ -155,8 +185,6 @@ static void c4x_usect
   PARAMS ((int));
 static void c4x_version
   PARAMS ((int));
-static void c4x_pseudo_ignore
-  PARAMS ((int));
 static void c4x_init_regtable
   PARAMS ((void));
 static void c4x_init_symbols
@@ -176,7 +204,9 @@ static int c4x_indirect_parse
 static char *c4x_operand_parse
   PARAMS ((char *, c4x_operand_t *));
 static int c4x_operands_match
-  PARAMS ((c4x_inst_t *, c4x_insn_t *));
+  PARAMS ((c4x_inst_t *, c4x_insn_t *, int));
+static void c4x_insn_check
+  PARAMS ((c4x_insn_t *));
 static void c4x_insn_output
   PARAMS ((c4x_insn_t *));
 static int c4x_operands_parse
@@ -1230,11 +1260,11 @@ c4x_version (x)
   input_line_pointer =
     c4x_expression_abs (input_line_pointer, &temp);
   if (!IS_CPU_C3X (temp) && !IS_CPU_C4X (temp))
-    as_bad ("This assembler does not support processor generation %d\n",
+    as_bad ("This assembler does not support processor generation %d",
 	    temp);
 
   if (c4x_cpu && temp != c4x_cpu)
-    as_warn ("Changing processor generation on fly not supported...\n");
+    as_warn ("Changing processor generation on fly not supported...");
   c4x_cpu = temp;
   demand_empty_rest_of_line ();
 }
@@ -1380,6 +1410,11 @@ c4x_inst_add (insts)
 
   d = name;
 
+  /* We do not care about INSNs that is not a part of our
+     oplevel setting */
+  if (!insts->oplevel & c4x_oplevel)
+    return ok;
+
   while (1)
     {
       switch (*s)
@@ -1441,6 +1476,34 @@ md_begin ()
   int ok = 1;
   unsigned int i;
 
+  /* Setup the proper opcode level according to the
+     commandline parameters */
+  c4x_oplevel = OP_C3X;
+
+  if ( IS_CPU_C4X(c4x_cpu) )
+    c4x_oplevel |= OP_C4X;
+
+  if ( (   c4x_cpu == 31 && c4x_revision >= 6)
+       || (c4x_cpu == 32 && c4x_revision >= 2)
+       || (c4x_cpu == 33)
+       || c4x_enhanced )
+    c4x_oplevel |= OP_ENH;
+
+  if ( (   c4x_cpu == 30 && c4x_revision >= 7)
+       || (c4x_cpu == 31 && c4x_revision >= 5)
+       || (c4x_cpu == 32)
+       || c4x_lowpower )
+    c4x_oplevel |= OP_LPWR;
+
+  if ( (   c4x_cpu == 30 && c4x_revision >= 7)
+       || (c4x_cpu == 31 && c4x_revision >= 5)
+       || (c4x_cpu == 32)
+       || (c4x_cpu == 33)
+       || (c4x_cpu == 40 && c4x_revision >= 5)
+       || (c4x_cpu == 44)
+       || c4x_idle2 )
+    c4x_oplevel |= OP_IDLE2;
+
   /* Create hash table for mnemonics.  */
   c4x_op_hash = hash_new ();
 
@@ -1448,14 +1511,8 @@ md_begin ()
   c4x_asg_hash = hash_new ();
 
   /* Add mnemonics to hash table, expanding conditional mnemonics on fly.  */
-  for (i = 0; i < c3x_num_insts; i++)
-    ok &= c4x_inst_add ((void *) &c3x_insts[i]);
-
-  if (IS_CPU_C4X (c4x_cpu))
-    {
-      for (i = 0; i < c4x_num_insts; i++)
-	ok &= c4x_inst_add ((void *) &c4x_insts[i]);
-    }
+  for (i = 0; i < c4x_num_insts; i++)
+    ok &= c4x_inst_add ((void *) &c4x_insts[i]);
 
   /* Create dummy inst to avoid errors accessing end of table.  */
   c4x_inst_make ("", 0, "");
@@ -1686,12 +1743,12 @@ c4x_operand_parse (s, operand)
 
     case '*':
       ret = -1;
-      for (i = 0; i < num_indirects; i++)
+      for (i = 0; i < c4x_num_indirects; i++)
 	if ((ret = c4x_indirect_parse (operand, &c4x_indirects[i])))
 	  break;
       if (ret < 0)
 	break;
-      if (i < num_indirects)
+      if (i < c4x_num_indirects)
 	{
 	  operand->mode = M_INDIRECT;
 	  /* Indirect addressing mode number.  */
@@ -1744,9 +1801,10 @@ c4x_operand_parse (s, operand)
 }
 
 static int 
-c4x_operands_match (inst, insn)
+c4x_operands_match (inst, insn, check)
      c4x_inst_t *inst;
      c4x_insn_t *insn;
+     int check;
 {
   const char *args = inst->args;
   unsigned long opcode = inst->opcode;
@@ -1792,9 +1850,20 @@ c4x_operands_match (inst, insn)
 	     use an immediate mode form of ldiu or ldpk instruction.  */
 	  if (exp->X_op == O_constant)
 	    {
-	      /* Maybe for C3x we should check for 8 bit number.  */
-	      INSERTS (opcode, exp->X_add_number, 15, 0);
-	      continue;
+              if( ( IS_CPU_C4X (c4x_cpu) && exp->X_add_number <= 65535 )
+                  || ( IS_CPU_C3X (c4x_cpu) && exp->X_add_number <= 255 ) )
+                {
+                  INSERTS (opcode, exp->X_add_number, 15, 0);
+                  continue;
+                }
+              else
+                {
+		  if (!check)
+                    as_bad ("Immediate value of %ld is too large for ldf",
+                            (long) exp->X_add_number);
+		  ret = -1;
+		  continue;
+                }
 	    }
 	  else if (exp->X_op == O_symbol)
 	    {
@@ -1808,10 +1877,10 @@ c4x_operands_match (inst, insn)
 	  if (operand->mode != M_DIRECT)
 	    break;
 	  if (exp->X_op == O_constant)
-	    {
-	      /* Store only the 16 LSBs of the number.  */
-	      INSERTS (opcode, exp->X_add_number, 15, 0);
-	      continue;
+            {
+              /* Store only the 16 LSBs of the number.  */
+              INSERTS (opcode, exp->X_add_number, 15, 0);
+              continue;
 	    }
 	  else if (exp->X_op == O_symbol)
 	    {
@@ -1829,7 +1898,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_AR0, 24, 22);
 	  else
 	    {
-	      as_bad ("Destination register must be ARn");
+              if (!check)
+                as_bad ("Destination register must be ARn");
 	      ret = -1;
 	    }
 	  continue;
@@ -1847,8 +1917,9 @@ c4x_operands_match (inst, insn)
 		}
 	      else
 		{
-		  as_bad ("Immediate value of %ld is too large",
-			  (long) exp->X_add_number);
+		  if (!check)
+                    as_bad ("Immediate value of %ld is too large",
+                            (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
 		}
@@ -1871,11 +1942,13 @@ c4x_operands_match (inst, insn)
 	    break;
 	  if (operand->mode != M_INDIRECT)
 	    break;
+	  /* Require either *+ARn(disp) or *ARn.  */
 	  if (operand->expr.X_add_number != 0
 	      && operand->expr.X_add_number != 0x18)
 	    {
-	      as_bad ("Invalid indirect addressing mode");
-	      ret = -1;
+              if (!check)
+                as_bad ("Invalid indirect addressing mode");
+              ret = -1;
 	      continue;
 	    }
 	  INSERTU (opcode, operand->aregno - REG_AR0, 2, 0);
@@ -1888,6 +1961,21 @@ c4x_operands_match (inst, insn)
 	  INSERTU (opcode, exp->X_add_number, 7, 0);
 	  continue;
 
+        case 'e':
+          if (!(operand->mode == M_REGISTER))
+            break;
+	  reg = exp->X_add_number;
+	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+               || (IS_CPU_C4X (c4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
+	    INSERTU (opcode, reg, 7, 0);
+	  else
+	    {
+              if (!check)
+                as_bad ("Register must be Rn");
+	      ret = -1;
+	    }
+          continue;
+
 	case 'F':
 	  if (operand->mode != M_IMMED_F
 	      && !(operand->mode == M_IMMED && exp->X_op == O_constant))
@@ -1913,6 +2001,21 @@ c4x_operands_match (inst, insn)
 	  INSERTU (opcode, exp->X_add_number, 15, 8);
 	  continue;
 
+        case 'g':
+	  if (operand->mode != M_REGISTER)
+	    break;
+	  reg = exp->X_add_number;
+	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+               || (IS_CPU_C4X (c4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
+	    INSERTU (opcode, reg, 15, 8);
+	  else
+	    {
+              if (!check)
+                as_bad ("Register must be Rn");
+	      ret = -1;
+	    }
+          continue;
+
 	case 'H':
 	  if (operand->mode != M_REGISTER)
 	    break;
@@ -1921,11 +2024,23 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_R0, 18, 16);
 	  else
 	    {
-	      as_bad ("Register must be R0--R7");
+              if (!check)
+                as_bad ("Register must be R0--R7");
 	      ret = -1;
 	    }
 	  continue;
 
+        case 'i':
+          if ( operand->mode == M_REGISTER
+               && c4x_oplevel & OP_ENH )
+            {
+              reg = exp->X_add_number;
+              INSERTU (opcode, reg, 4, 0);
+              INSERTU (opcode, 7, 7, 5);
+              continue;
+            }
+          /* Fallthrough */
+
 	case 'I':
 	  if (operand->mode != M_INDIRECT)
 	    break;
@@ -1933,8 +2048,9 @@ c4x_operands_match (inst, insn)
 	    {
 	      if (IS_CPU_C4X (c4x_cpu))
 		break;
-	      as_bad ("Invalid indirect addressing mode displacement %d",
-		      operand->disp);
+              if (!check)
+                as_bad ("Invalid indirect addressing mode displacement %d",
+                        operand->disp);
 	      ret = -1;
 	      continue;
 	    }
@@ -1942,6 +2058,17 @@ c4x_operands_match (inst, insn)
 	  INSERTU (opcode, operand->expr.X_add_number, 7, 3);
 	  continue;
 
+        case 'j':
+          if ( operand->mode == M_REGISTER
+               && c4x_oplevel & OP_ENH )
+            {
+              reg = exp->X_add_number;
+              INSERTU (opcode, reg, 12, 8);
+              INSERTU (opcode, 7, 15, 13);
+              continue;
+            }
+          /* Fallthrough */
+
 	case 'J':
 	  if (operand->mode != M_INDIRECT)
 	    break;
@@ -1949,8 +2076,9 @@ c4x_operands_match (inst, insn)
 	    {
 	      if (IS_CPU_C4X (c4x_cpu))
 		break;
-	      as_bad ("Invalid indirect addressing mode displacement %d",
-		      operand->disp);
+              if (!check)
+                as_bad ("Invalid indirect addressing mode displacement %d",
+                        operand->disp);
 	      ret = -1;
 	      continue;
 	    }
@@ -1966,7 +2094,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_R0, 21, 19);
 	  else
 	    {
-	      as_bad ("Register must be R0--R7");
+              if (!check)
+                as_bad ("Register must be R0--R7");
 	      ret = -1;
 	    }
 	  continue;
@@ -1979,7 +2108,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_R0, 24, 22);
 	  else
 	    {
-	      as_bad ("Register must be R0--R7");
+              if (!check)
+                as_bad ("Register must be R0--R7");
 	      ret = -1;
 	    }
 	  continue;
@@ -1992,7 +2122,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_R2, 22, 22);
 	  else
 	    {
-	      as_bad ("Destination register must be R2 or R3");
+              if (!check)
+                as_bad ("Destination register must be R2 or R3");
 	      ret = -1;
 	    }
 	  continue;
@@ -2005,7 +2136,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_R0, 23, 23);
 	  else
 	    {
-	      as_bad ("Destination register must be R0 or R1");
+              if (!check)
+                as_bad ("Destination register must be R0 or R1");
 	      ret = -1;
 	    }
 	  continue;
@@ -2019,7 +2151,8 @@ c4x_operands_match (inst, insn)
 	  if (operand->expr.X_add_number != 0
 	      && operand->expr.X_add_number != 0x18)
 	    {
-	      as_bad ("Invalid indirect addressing mode");
+              if (!check)
+                as_bad ("Invalid indirect addressing mode");
 	      ret = -1;
 	      continue;
 	    }
@@ -2040,8 +2173,9 @@ c4x_operands_match (inst, insn)
 		}
 	      else
 		{
-		  as_bad ("Displacement value of %ld is too large",
-			  (long) exp->X_add_number);
+                  if (!check)
+                    as_bad ("Displacement value of %ld is too large",
+                            (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
 		}
@@ -2058,6 +2192,21 @@ c4x_operands_match (inst, insn)
 	  INSERTU (opcode, reg, 15, 0);
 	  continue;
 
+        case 'q':
+	  if (operand->mode != M_REGISTER)
+	    break;
+	  reg = exp->X_add_number;
+	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+               || (IS_CPU_C4X (c4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
+	    INSERTU (opcode, reg, 15, 0);
+	  else
+	    {
+              if (!check)
+                as_bad ("Register must be Rn");
+	      ret = -1;
+	    }
+          continue;
+
 	case 'R':
 	  if (operand->mode != M_REGISTER)
 	    break;
@@ -2065,12 +2214,28 @@ c4x_operands_match (inst, insn)
 	  INSERTU (opcode, reg, 20, 16);
 	  continue;
 
+        case 'r':
+	  if (operand->mode != M_REGISTER)
+	    break;
+	  reg = exp->X_add_number;
+	  if ( (reg >= REG_R0 && reg <= REG_R7) 
+               || (IS_CPU_C4X (c4x_cpu) && reg >= REG_R8 && reg <= REG_R11) )
+	    INSERTU (opcode, reg, 20, 16);
+	  else
+	    {
+              if (!check)
+                as_bad ("Register must be Rn");
+	      ret = -1;
+	    }
+          continue;
+
 	case 'S':		/* Short immediate int.  */
 	  if (operand->mode != M_IMMED && operand->mode != M_HI)
 	    break;
 	  if (exp->X_op == O_big)
 	    {
-	      as_bad ("Floating point number not valid in expression");
+              if (!check)
+                as_bad ("Floating point number not valid in expression");
 	      ret = -1;
 	      continue;
 	    }
@@ -2083,8 +2248,9 @@ c4x_operands_match (inst, insn)
 		}
 	      else
 		{
-		  as_bad ("Signed immediate value %ld too large",
-			  (long) exp->X_add_number);
+		  if (!check)
+                    as_bad ("Signed immediate value %ld too large",
+                            (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
 		}
@@ -2124,8 +2290,9 @@ c4x_operands_match (inst, insn)
 		}
 	      else
 		{
-		  as_bad ("Immediate value of %ld is too large",
-			  (long) exp->X_add_number);
+                  if (!check)
+                    as_bad ("Immediate value of %ld is too large",
+                            (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
 		}
@@ -2144,8 +2311,9 @@ c4x_operands_match (inst, insn)
 		}
 	      else
 		{
-		  as_bad ("Unsigned immediate value %ld too large",
-			  (long) exp->X_add_number);
+                  if (!check)
+                    as_bad ("Unsigned immediate value %ld too large",
+                            (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
 		}
@@ -2181,8 +2349,9 @@ c4x_operands_match (inst, insn)
 		}
 	      else
 		{
-		  as_bad ("Immediate value of %ld is too large",
-			  (long) exp->X_add_number);
+                  if (!check)
+                    as_bad ("Immediate value of %ld is too large",
+                            (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
 		}
@@ -2196,7 +2365,8 @@ c4x_operands_match (inst, insn)
 	    break;
 	  if (exp->X_op == O_big)
 	    {
-	      as_bad ("Floating point number not valid in expression");
+              if (!check)
+                as_bad ("Floating point number not valid in expression");
 	      ret = -1;
 	      continue;
 	    }
@@ -2209,8 +2379,9 @@ c4x_operands_match (inst, insn)
 		}
 	      else
 		{
-		  as_bad ("Immediate value %ld too large",
-			  (long) exp->X_add_number);
+                  if (!check)
+                    as_bad ("Immediate value %ld too large",
+                            (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
 		}
@@ -2227,7 +2398,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_IVTP, 4, 0);
 	  else
 	    {
-	      as_bad ("Register must be ivtp or tvtp");
+              if (!check)
+                as_bad ("Register must be ivtp or tvtp");
 	      ret = -1;
 	    }
 	  continue;
@@ -2240,7 +2412,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg, 20, 16);
 	  else
 	    {
-	      as_bad ("Register must be address register");
+              if (!check)
+                as_bad ("Register must be address register");
 	      ret = -1;
 	    }
 	  continue;
@@ -2253,7 +2426,8 @@ c4x_operands_match (inst, insn)
 	    INSERTU (opcode, reg - REG_IVTP, 20, 16);
 	  else
 	    {
-	      as_bad ("Register must be ivtp or tvtp");
+              if (!check)
+                as_bad ("Register must be ivtp or tvtp");
 	      ret = -1;
 	    }
 	  continue;
@@ -2300,6 +2474,38 @@ c4x_operands_match (inst, insn)
     }
 }
 
+static void
+c4x_insn_check (insn)
+     c4x_insn_t *insn;
+{
+  
+  if (!strcmp(insn->name, "lda"))
+    {
+      if (insn->num_operands < 2 || insn->num_operands > 2)
+        as_fatal ("Illegal internal LDA insn definition");
+
+      if ( insn->operands[0].mode == M_REGISTER
+           && insn->operands[1].mode == M_REGISTER
+           && insn->operands[0].expr.X_add_number == insn->operands[1].expr.X_add_number )
+        as_bad ("Source and destination register should not be equal");
+    }
+  else if( !strcmp(insn->name, "ldi_ldi")
+           || !strcmp(insn->name, "ldi1_ldi2")
+           || !strcmp(insn->name, "ldi2_ldi1")
+           || !strcmp(insn->name, "ldf_ldf")
+           || !strcmp(insn->name, "ldf1_ldf2")
+           || !strcmp(insn->name, "ldf2_ldf1") )
+    {
+      if ( insn->num_operands < 4 && insn->num_operands > 5 )
+        as_fatal ("Illegal internal %s insn definition", insn->name);
+      
+      if ( insn->operands[1].mode == M_REGISTER
+           && insn->operands[insn->num_operands-1].mode == M_REGISTER
+           && insn->operands[1].expr.X_add_number == insn->operands[insn->num_operands-1].expr.X_add_number )
+        as_warn ("Equal parallell destination registers, one result will be discarded");
+    }
+}
+
 static void 
 c4x_insn_output (insn)
      c4x_insn_t *insn;
@@ -2359,6 +2565,7 @@ md_assemble (str)
   int i;
   int parsed = 0;
   c4x_inst_t *inst;		/* Instruction template.  */
+  c4x_inst_t *first_inst;
 
   if (str && insn->parallel)
     {
@@ -2405,23 +2612,30 @@ md_assemble (str)
 	  return;
 	}
 
-      /* FIXME:  The list of templates should be scanned
-         for the candidates with the desired number of operands.
-         We shouldn't issue error messages until we have
-         whittled the list of candidate templates to the most
-         likely one...  We could cache a parsed form of the templates
-         to reduce the time required to match a template.  */
-
       inst = insn->inst;
-
+      first_inst = NULL;
       do
-	ok = c4x_operands_match (inst, insn);
-      while (!ok && !strcmp (inst->name, inst[1].name) && inst++);
+        {
+          ok = c4x_operands_match (inst, insn, 1);
+          if (ok < 0)
+            {
+              if (!first_inst)
+                first_inst = inst;
+              ok = 0;
+            }
+      } while (!ok && !strcmp (inst->name, inst[1].name) && inst++);
 
       if (ok > 0)
-	c4x_insn_output (insn);
+        {
+          c4x_insn_check (insn);
+          c4x_insn_output (insn);
+        }
       else if (!ok)
-	as_bad ("Invalid operands for %s", insn->name);
+        {
+          if (first_inst)
+            c4x_operands_match (first_inst, insn, 0);
+          as_bad ("Invalid operands for %s", insn->name);
+        }
       else
 	as_bad ("Invalid instruction %s", insn->name);
     }
@@ -2629,13 +2843,6 @@ md_estimate_size_before_relax (fragP, segtype)
   return 0;
 }
 
-CONST char *md_shortopts = "bm:prs";
-struct option md_longopts[] =
-{
-  {NULL, no_argument, NULL, 0}
-};
-
-size_t md_longopts_size = sizeof (md_longopts);
 
 int
 md_parse_option (c, arg)
@@ -2644,25 +2851,54 @@ md_parse_option (c, arg)
 {
   switch (c)
     {
-    case 'b':			/* big model */
-      c4x_big_model = 1;
-      break;
-    case 'm':			/* -m[c][34]x */
+    case OPTION_CPU:             /* cpu brand */
       if (tolower (*arg) == 'c')
 	arg++;
       c4x_cpu = atoi (arg);
       if (!IS_CPU_C3X (c4x_cpu) && !IS_CPU_C4X (c4x_cpu))
-	as_warn ("Unsupported processor generation %d\n", c4x_cpu);
+	as_warn ("Unsupported processor generation %d", c4x_cpu);
       break;
-    case 'p':			/* push args */
+
+    case OPTION_REV:             /* cpu revision */
+      c4x_revision = atoi (arg);
+      break;
+
+    case 'b':
+      as_warn ("Option -b is depreciated, please use -mbig");
+    case OPTION_BIG:             /* big model */
+      c4x_big_model = 1;
+      break;
+
+    case 'p':
+      as_warn ("Option -p is depreciated, please use -mmemparm");
+    case OPTION_MEMPARM:         /* push args */
       c4x_reg_args = 0;
       break;
-    case 'r':			/* register args */
+
+    case 'r':			
+      as_warn ("Option -r is depreciated, please use -mregparm");
+    case OPTION_REGPARM:        /* register args */
       c4x_reg_args = 1;
       break;
-    case 's':			/* small model */
+
+    case 's':
+      as_warn ("Option -s is depreciated, please use -msmall");
+    case OPTION_SMALL:		/* small model */
       c4x_big_model = 0;
       break;
+
+    case OPTION_IDLE2:
+      c4x_idle2 = 1;
+      break;
+
+    case OPTION_LOWPOWER:
+      c4x_lowpower = 1;
+      break;
+
+    case OPTION_ENHANCED:
+      c4x_enhanced = 1;
+      break;
+
     default:
       return 0;
     }
@@ -2674,15 +2910,26 @@ void
 md_show_usage (stream)
      FILE *stream;
 {
-  fputs ("\
-C[34]x options:\n\
--m30 | -m31 | -m32 | -m33 | -m40 | -m44\n\
-			specify variant of architecture\n\
--b                      big memory model\n\
--p                      pass arguments on stack\n\
--r                      pass arguments in registers (default)\n\
--s                      small memory model (default)\n",
-	 stream);
+  fprintf (stream,
+      _("\nTIC4X options:\n"
+	"  -mcpu=CPU  -mCPU        select architecture variant. CPU can be:\n"
+	"                            30 - TMS320C30\n"
+	"                            31 - TMS320C31, TMS320LC31\n"
+	"                            32 - TMS320C32\n"
+        "                            33 - TMS320VC33\n"
+	"                            40 - TMS320C40\n"
+	"                            44 - TMS320C44\n"
+        "  -mrev=REV               set cpu hardware revision (integer numbers).\n"
+        "                          Combinations of -mcpu and -mrev will enable/disable\n"
+        "                          the appropriate options (-midle2, -mlowpower and\n"
+        "                          -menhanced) according to the selected type\n"
+        "  -mbig                   select big memory model\n"
+        "  -msmall                 select small memory model (default)\n"
+        "  -mregparm               select register parameters (default)\n"
+        "  -mmemparm               select memory parameters\n"
+        "  -midle2                 enable IDLE2 support\n"
+        "  -mlowpower              enable LOPOWER and MAXSPEED support\n"
+        "  -menhanced              enable enhanced opcode support\n"));
 }
 
 /* This is called when a line is unrecognized.  This is used to handle
@@ -2924,7 +3171,7 @@ tc_gen_reloc (seg, fixP)
   if (reloc->howto == (reloc_howto_type *) NULL)
     {
       as_bad_where (fixP->fx_file, fixP->fx_line,
-		    "reloc %d not supported by object file format",
+		    "Reloc %d not supported by object file format",
 		    (int) fixP->fx_r_type);
       return NULL;
     }
