@@ -5,7 +5,7 @@ GENERATE_SHLIB_SCRIPT=yes
 SCRIPT_NAME=elf
 OUTPUT_FORMAT="elf64-powerpc"
 TEXT_START_ADDR=0x10000000
-SEGMENT_SIZE=0x10000000
+#SEGMENT_SIZE=0x10000000
 MAXPAGESIZE=0x10000
 COMMONPAGESIZE=0x1000
 ARCH=powerpc:common64
